package	jp.co.sds_corp.syslog;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketTimeoutException;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import jp.co.sds_corp.syslog.action.IAction;
import jp.co.sds_corp.syslog.filter.FilterUtility;
import jp.co.sds_corp.syslog.filter.IFilter;
import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.AbstractService;
import jp.co.sds_corp.util.daemon.config.ConfigUtility;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.syslog.SyslogMsg;

/**
* SyslogViewerの動作をするメインクラスです。
*/
public class SyslogService extends AbstractService
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	transient DatagramSocket socRead;
	int	port;
	static SyslogService instance;
	Vector<ISyslogListener>	vecLsn;
	Vector<IAction>	vecAct;

	/**
	* 初期化されたインスタンスを返します。
	* @return	このサービスのインスタンス
	*/
	public static SyslogService getInstance() {
		return instance;
	}

	/**
	* 指定された初期情報で、初期化します。
	* @param    config  初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		vecLsn = new Vector<ISyslogListener>();
		vecAct = new Vector<IAction>();
		port = config.getInteger( "port", 514 );

		Iterator<IConfiguration> itr = config.getChilds( "Action" );
		while ( itr.hasNext() ) {
			IAction act = createAction( (IConfiguration )itr.next() );
			vecAct.add( act );
		}

		logger.debug( this, "start receive. port=" + port );
		socRead = new DatagramSocket( port );
		socRead.setSoTimeout( 1000 );
		instance = this;
	}

	protected IAction createAction( IConfiguration config )
		throws Exception
	{
		if ( config == null ) {
			return null;
		}
		IAction ret;
		ret = (IAction )ConfigUtility.createInstance( config );
		ret.init( config );
		return ret;
	}

	/**
	* サービスを開始します。
	*/
	public void start() {
		byte[]  aByte = new byte[60000];
		DatagramPacket packet = new DatagramPacket( aByte, aByte.length );

		for ( ;; ) {
			if ( isInterrupted()) {
				break;
			}
			try {
				socRead.receive( packet );
				if ( packet.getLength() <= 0 ) {
					continue;
				}
				Thread.yield();
				SyslogMsg msg = new SyslogMsg( aByte, packet.getLength() );
				msg.setAddress( packet.getAddress() );
				msg.setDate( new CmnDate() );
				SyslogEvent ev = new SyslogEvent( this, msg );
				fireSyslogReceive( ev );
/*
				logger.debug( this,
					msg.getDate().format( "yyyy/MM/dd HH:mm:ss.SSS" ) + " " +
					msg.getAddress().getHostName() + " " +
					msg.getFacility() + " " +
					msg.getPriority() + " " +
					msg.getMessage() );
*/
			} catch( SocketTimeoutException ste ) {
				// 無視
			} catch( Exception ex ) {
				logger.error( this, "error", ex );
			}
		}
	}

	/**
	* このサービスを停止します。
	*/
	public void stop() {
	}

	/**
	* {@link IAction}の一覧を返します。
	* @return	{@link IAction}の一覧
	*/
	public List<IAction> getActionList() {
		return vecAct;
	}

	/**
	* syslogの受信イベントを受け取るオブジェクトを追加します。
	* @param	l		syslogの受信イベントを受け取るオブジェクト
	*/
	public void addSyslogListener( ISyslogListener l ) {
		vecLsn.add( l );
	}
	/**
	* syslogの受信イベントを受け取るオブジェクトを削除します。
	* @param    l       syslogの受信イベントを受け取るオブジェクト
	*/
	public void removeSyslogListener( ISyslogListener l ) {
		vecLsn.remove( l );
	}

	/**
	* syslog受信イベントを通知します。
	* @param	ev		受信イベント
	*/
	public void fireSyslogReceive( SyslogEvent ev ) {
		for ( int i = 0; i < vecAct.size(); i++ ) {
			IAction act = (IAction )vecAct.get( i );
			if ( !act.isValid()) {
				continue;
			}

			List<IFilter> l = act.listFilters();
			boolean bFilter = FilterUtility.isFiltered( l, ev );
			if ( !bFilter ) {
				try {
					act.action( ev );
				} catch ( Exception ex ) {
					logger.error( this, "error", ex );
				}
			}
		}
		for ( int i = 0; i < vecLsn.size(); i++ ) {
			try {
				ISyslogListener l = (ISyslogListener )vecLsn.get( i );
				l.syslogReceived( ev );
			} catch ( Exception e ) {
				logger.error( this, "error", e );
			}
		}
	}

	/**
	* このサービスの文字列表現を返します。
	*/
	public String toString() {
		StringBuilder buf = new StringBuilder();
		buf.append( super.toString() + " port=" + port );
		if ( vecAct != null ) {
			for ( int i = 0; i < vecAct.size(); i++ ) {
				IAction act = (IAction )vecAct.get( i );
				buf.append( "\n\t" + act.toString());
			}
		}
		return buf.toString();
	}

	/**
	* 設定されている内容を出力します。
	* @param    config  設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		config.addChild( "port", String.valueOf( port ));

		if ( vecAct != null ) {
			for ( int i = 0; i < vecAct.size(); i++ ) {
				IAction act = (IAction )vecAct.get( i );
				IConfiguration c = config.addChild( "Action" );
				act.saveConfig( c );
			}
		}
	}

}

