package	jp.co.sds_corp.syslog.filter;

import jp.co.sds_corp.syslog.SyslogEvent;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.syslog.Facility;
import jp.co.sds_corp.util.syslog.Priority;
import jp.co.sds_corp.util.syslog.SyslogMsg;

/**
* syslogのFacilityとPriorityによりフィルタリングを行ないます。
XMLは以下の形式です。
<TABLE BORDER="1">
	<TR><TH>要素</TH><TH>必須</TH><TH>説明</TH><TH>備考</TH></TR>
	<TR>
		<TD>Facility</TD>
		<TD>&nbsp;</TD>
		<TD>指定されたFacility以外をフィルタ</TD>
		<TD>&nbsp;</TD>
	</TR>
	<TR>
		<TD>MaxPriority</TD>
		<TD>&nbsp;</TD>
		<TD>最大のPriorityを指定</TD>
		<TD>&nbsp;</TD>
	</TR>
	<TR>	
		<TD>MinPriority</TD>
		<TD>&nbsp;</TD>
		<TD>最小のPriorityを指定</TD>
		<TD>&nbsp;</TD>
	</TR>
	<TR>
		<TD>Exclude</TD>
		<TD>&nbsp;</TD>
		<TD>除外フラグ
			(true:条件に合致すれば除外/false:条件に合致するもののみ対象)</TD>
		<TD>指定されない場合はfalse</TD>
	</TR>
	<TR>
		<TD>Valid</TD>
		<TD>&nbsp;</TD>
		<TD>有効フラグ(true:有効/false:無効)</TD>
		<TD>指定されない場合はtrue</TD>
	</TR>
</TABLE>
<PRE>
&lt;Filter class="jp.co.sds_corp.syslog.filter.LevelFilter"&gt;
	&lt;Facility&gt;USER&lt;/Facility&gt;
	&lt;MaxPriority&gt;ERR&lt;/MaxPriority&gt;
	&lt;MinPriority&gt;INFO&lt;/MinPriority&gt;
	&lt;Exclude&gt;true&lt;/Exclude&gt;
	&lt;Valid&gt;false&lt;/Valid&gt;
&lt;/Filter&gt;
</PRE>
*/
public class LevelFilter extends AbstractFilter
{
	private static final long serialVersionUID = 1L;
	Facility	facility;
	Priority	maxPriority;
	Priority	minPriority;

	/**
	* 指定された初期情報で初期化します。
	* @param    config  初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		String var;
		var = config.get( "Facility", null );
		if ( var != null ) {
			facility = Facility.parse( var );
		}
		var = config.get( "MaxPriority", null );
		if ( var != null ) {
			maxPriority = Priority.parse( var );
		}
		var = config.get( "MinPriority", null );
		if ( var != null ) {
			minPriority = Priority.parse( var );
		}
	}

	/**
	* syslogのがこのフィルタの条件に合致するかを検査します。
	* @param	e		データ受信イベント
	*/
	public boolean match( SyslogEvent e ) {
		SyslogMsg msg = e.getMessage();
		if ( facility != null ) {
			if ( !facility.equals( msg.getFacility()) ) {
				return false;
			}
		}
		if ( minPriority != null ) {
			if ( minPriority.compareTo( msg.getPriority() ) < 0 ) {
				return false;
			}
		}
		if ( maxPriority != null ) {
			if ( maxPriority.compareTo( msg.getPriority() ) > 0 ) {
				return false;
			}
		}
		return true;
	}

	/**
	* 設定内容の文字列表現を返します。
	*/
	public String paramString() {
		String ret = "{";
		ret += super.paramString();
		boolean bAdd = false;
		if ( facility != null ) {
			ret += "F=" + facility;
			bAdd = true;
		}
		if ( maxPriority != null ) {
			if ( bAdd ) {
				ret += " ";
			}
			ret += "Max=" + maxPriority;
			bAdd = true;
		}
		if ( minPriority != null ) {
			if ( bAdd ) {
				ret += " ";
			}
			ret += "Min=" + minPriority;
			bAdd = true;
		}
		ret += "}";
		return ret;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	親ノード
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		IConfiguration c;
		if ( getFacility() != null ) {
			c = config.addChild( "Facility", getFacility().toString() );
		}
		if ( getMaxPriority() != null ) {
			c = config.addChild( "MaxPriority", getMaxPriority().toString() );
		}
		if ( getMinPriority() != null ) {
			c = config.addChild( "MinPriority", getMinPriority().toString() );
		}
	}

	/**
	* 種別(facility)を設定します。
	* @param	var		設定するデータ
	*/
	public void setFacility( Facility var ) {
		this.facility = var ;
	}
	/**
	* 種別(facility)を取得します。
	* @return	facilityの内容
	*/
	public Facility getFacility() {
		return facility;
	}

	/**
	* 優先度(maxPriority)を設定します。
	* @param	var		設定するデータ
	*/
	public void setMaxPriority( Priority var ) {
		this.maxPriority = var ;
	}
	/**
	* 優先度(maxPriority)を取得します。
	* @return	maxPriorityの内容
	*/
	public Priority getMaxPriority() {
		return maxPriority;
	}

	/**
	* 優先度(minPriority)を設定します。
	* @param	var		設定するデータ
	*/
	public void setMinPriority( Priority var ) {
		this.minPriority = var ;
	}
	/**
	* 優先度(minPriority)を取得します。
	* @return	minPriorityの内容
	*/
	public Priority getMinPriority() {
		return minPriority;
	}

}
